using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Web.UI;
//using System.Web.UI.Design;
//using System.Web.UI.Design.WebControls;
using System.Web.UI.WebControls;

//using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a table that contains a single rows. This helps to simplify the creation of
	/// consistent place holders for adhoc ribbon layouts.
	/// </summary>
	[
	//Designer(typeof(OneRowLayoutDesigner)),
	ToolboxData("<{0}:OneRowLayout runat=\"server\"></{0}:OneRowLayout>"),
	ParseChildren(true)
	]
	public class OneRowLayout : MyWebControl
	{

		#region Fields

		private ITemplate _placeHolder = null;
		private ControlCollection _placeHolderControls = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializises a new instance of the OneRowLayout class.
		/// </summary>
		public OneRowLayout()
		{
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the template content for the control's place holder.
		/// </summary>
		[
		Browsable(false),
		PersistenceMode(PersistenceMode.InnerProperty), DefaultValue(null),
		//TemplateInstance(TemplateInstance.Single)
		]
		public virtual ITemplate PlaceHolder
		{
			get
			{
				return _placeHolder;
			}
			set
			{
				_placeHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the place holder at run-time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection PlaceHolderControls
		{
			get
			{
				if (_placeHolderControls == null)
				{
					_placeHolderControls = new ControlCollection(this);
				}

				return _placeHolderControls;
			}
		}

		#endregion Properties

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Ensure child controls are available for referencing in the page load.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			EnsureChildControls();
		}

		/// <summary>
		/// Ensure we add runtime controls so they are available for event handler re-attachment.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			if (!this.DesignMode)
			{
				AddRuntimeControls();
			}
		}

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			this.Controls.Clear();

			this.Attributes.Add("class", "ribbonGroupBlock");

			Table table = new Table();
			table.CellSpacing = 0;
			table.CellPadding = 0;

			TableRow row = new TableRow();
			
			TableCell cell = new TableCell();
			cell.Attributes.Add("class", "ribbonGroupBlockFull ribbonPart");

			row.Cells.Add(cell);

			if (this.PlaceHolder != null)
			{
				this.PlaceHolder.InstantiateIn(cell);
			}

			table.Rows.Add(row);

			if (this.DesignMode)
			{
				table.Width = 8;
			}

			this.Controls.Add(table);
		}

		/// <summary>
		/// Adds controls added at run-time before rendering.
		/// </summary>
		private void AddRuntimeControls()
		{
			Table table = this.Controls[0] as Table;
			TableCell cell = table.Rows[0].Cells[0];

			foreach (Control control in this.PlaceHolderControls)
			{
				cell.Controls.Add(control);
			}
		}

		#endregion Implementation Methods

	}

}

