using System;
using System.Collections;
using System.Text;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.ComponentModel;

//using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a quick access button displayed on the main apllication bar.
	/// </summary>
	[
	//Designer(typeof(RibbonButtonDesigner)),
	ToolboxData("<{0}:QuickAccessRibbonButton runat=\"server\"></{0}:QuickAccessRibbonButton>")
	]
	public class QuickAccessRibbonButton : RibbonButton
	{

		#region Contructors

		/// <summary>
		/// Initializes a new instance of the QuickAccessRibbonButton class.
		/// </summary>
		public QuickAccessRibbonButton() : this(string.Empty)
		{
			// Shared State
			// 0 - Enabled
			// 1 - Checked
			// 2 - EnabledImageUrl
			// 3 - DisabledImageUrl
		}

		/// <summary>
		/// Initializes a new instance of the QuickAccessRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		public QuickAccessRibbonButton(string id) : this(id, string.Empty, string.Empty)
		{
		}

		/// <summary>
		/// Initializes a new instance of the QuickAccessRibbonButton class.
		/// </summary>
		/// <param name="id">
		/// The ID to assign to the control.
		/// </param>
		/// <param name="enabledImageUrl">
		/// The URL to an image to display when enabled.
		/// </param>
		/// <param name="disabledImageUrl">
		/// The URL to an image to display when disabled.
		/// </param>
		public QuickAccessRibbonButton(string id, string enabledImageUrl, string disabledImageUrl)
		{
			this.ID = id;
			this.EnabledImageUrl = enabledImageUrl;
			this.DisabledImageUrl = disabledImageUrl;
		}

		#endregion Contructors

		#region Implementation Methods

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			ClearControls();

			this.Attributes.Add("title", this.ToolTip);

			RenderClientPostBack();

			HtmlImage image = new HtmlImage();
			image.EnableViewState = false;
			image.ID = "Image";

			this.Controls.Add(image);
		}

		/// <summary>
		/// Sets the button's checked state.
		/// </summary>
		protected override void SetCheckedState()
		{
			if (this.Checked)
			{
				this.Attributes["class"] = "quickAccessButtonChecked";
			}
			else
			{
				this.Attributes["class"] = "quickAccessButton";
			}
		}

		#endregion Implementation Methods

	}

}
