using System;
using System.Collections;
using System.Text;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;

using Telerik.WebControls;

//using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Managers the layout of the ribbon such as expanded and collapsed Ribbon Groups.
	/// </summary>
	[
	//Designer(typeof(RibbonResizingManagerDesigner)),
	ToolboxData("<{0}:RibbonResizingManager runat=\"server\"></{0}:RibbonResizingManager>")
	]
	public class RibbonResizingManager : RibbonControl
	{

		#region Fields

		private string _ribbonTabStripID = string.Empty;

		#endregion Fields

		#region Constructors
		
		/// <summary>
		/// Initializes a new instance of the RibbonResizingManager class.
		/// </summary>
		public RibbonResizingManager()
		{
			// Shared State
			// 0 - ApplicationBarID
			// 1 - ApplicationMenuID
			// 2 - RibbonTabStripID
			// 3 - RibbonMultiPageViewID
			// 4 - MinimumRibbonWidth
			// 5 - RibbonHidden
			// 6 - ControlSet - IDCount, IDList, IDCount, IDList
			this.SharedState.Add(new SharedStateItem("ApplicationBarID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("ApplicationMenuID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("RibbonTabStripID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("RibbonMultiPageViewID", typeof(string), string.Empty));
			this.SharedState.Add(new SharedStateItem("MinimumRibbonWidth", typeof(int), 0));
			this.SharedState.Add(new SharedStateItem("RibbonHidden", typeof(bool), false));
			this.SharedState.Add(new SharedStateItem("ControlSet", typeof(string), string.Empty));
		}

		#endregion Constructors

		#region Properties 

		/// <summary>
		/// Gets or sets the minimum width (in pixels) of the Ribbon before it is hidden.
		/// </summary>
		[
		DefaultValue(0),
		Description("Specifies the minimum width (in pixels) of the Ribbon before it is hidden."),
		Category("Behavior")
		]
		public virtual int MinimumRibbonWidth
		{
			get
			{
				int minimumWidth = Convert.ToInt32(this.SharedState["MinimumRibbonWidth"].Value);

				return minimumWidth;
			}
			set
			{
				this.SharedState["MinimumRibbonWidth"].Value = value;
			}
		}

		#endregion Properties

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		/// <summary>
		/// Gets whether the ribbon is hidden because the width is too small. This is for
		/// internal use only, intended to reflect the client-side value only.
		/// </summary>
		protected virtual bool RibbonHidden
		{
			get
			{
				bool hidden = Convert.ToBoolean(this.SharedState["RibbonHidden"].Value);

				return hidden;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods
		
		/// <summary>
		/// Sets the originally selected tab prior to saving shared state.
		/// </summary>
		protected override void SaveSharedState()
		{
			if (this.Page != null)
			{
				// Calculate before saving state as the controls may not be available when the 
				// properties were set

				RibbonControlFinder finder = new RibbonControlFinder(this.Page);

				ApplicationBar applicationBar = finder.FindApplicationBar();
				RadMenu applicationMenu = finder.FindApplicationMenu();
				RadTabStrip tabStrip = finder.FindRibbonTabStrip();

				string clientId = GetClientID(this.Page, applicationBar.ID);
				this.SharedState["ApplicationBarID"].Value = clientId;

				clientId = GetClientID(this.Page, applicationMenu.ID);
				this.SharedState["ApplicationMenuID"].Value = clientId;

				clientId = GetClientID(this.Page, tabStrip.ID);
				this.SharedState["RibbonTabStripID"].Value = clientId;

				this.SharedState["RibbonMultiPageViewID"].Value = tabStrip.MultiPage.ClientID;

				// Format: {Number of groups, Expanded width, Collapsed width} x Number of tabs
				// Listed in order of collaspe sequence
				string controlSet = string.Empty;

				if ((tabStrip != null) && (tabStrip.MultiPage != null))
				{
					foreach (PageView pageView in tabStrip.MultiPage.PageViews)
					{
						IList groupList = new ArrayList();
						controlSet += (controlSet == string.Empty ? string.Empty : ",") + "{0}";

						foreach (Control control in pageView.Controls)
						{
							RibbonGroup group = control as RibbonGroup;

							if (group != null)
							{
								groupList.Add(group);
							}
						}

						// Sort in assending order based on CollapseSequence ...
//						groupList.Sort(new Comparison<RibbonGroup>(delegate(RibbonGroup group1, RibbonGroup group2) 
//							{
//								int compare = group1.CollapseSequence.CompareTo(group2.CollapseSequence);
//
//								// ... but if the same sequence is given use right to left as a sequence
//								if (compare == 0)
//								{
//									int indexInParent1 = group1.Parent.Controls.IndexOf(group1);
//									int indexInParent2 = group2.Parent.Controls.IndexOf(group2);
//
//									compare = indexInParent2.CompareTo(indexInParent1);
//								}
//
//								return compare;
//							}));

						const int leftMargin = 3;
						const int gap = 3;
						const int rightMargin = 3;

						// Shared state for each group
						for (int groupIndex = 0; groupIndex < groupList.Count; groupIndex++)
						{
							RibbonGroup group = groupList[groupIndex] as RibbonGroup;
							int transitionWidth = 0;

							for (int calculationIndex = 0; calculationIndex < groupList.Count; calculationIndex++)
							{
								RibbonGroup calculationGroup = groupList[calculationIndex] as RibbonGroup;

								if (calculationIndex < groupIndex)
								{
									transitionWidth += calculationGroup.CollapsedWidth;
								}
								else 
								{
									transitionWidth += calculationGroup.ExpandedWidth;
								}
							}

							transitionWidth += leftMargin + (gap * (groupList.Count - 1)) + rightMargin;

							controlSet += ",\"" + group.ClientMemberName + "\"" +
								"," + group.ExpandedWidth.ToString() +
								"," + group.CollapsedWidth.ToString() +
								"," + transitionWidth.ToString();
						}

						controlSet = string.Format(controlSet, groupList.Count.ToString());
					}
				}

				controlSet = "[" + controlSet + "]";

				this.SharedState["ControlSet"].Value = controlSet;
			}

			base.SaveSharedState();
		}

		#endregion Implementation Methods

	}

}
