using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing;
using System.Web.UI;
//using System.Web.UI.Design;
//using System.Web.UI.Design.WebControls;
using System.Web.UI.WebControls;

//using RussellMason.Web.UI.Office2007.Design;

namespace RussellMason.Web.UI.Office2007
{

	/// <summary>
	/// Represents a table that contains 2 rows. This helps to simplify the creation of
	/// consistent place holders for ribbon toolbars or other layouts.
	/// </summary>
	[
	//Designer(typeof(TwoRowLayoutDesigner)),
	ToolboxData("<{0}:TwoRowLayout runat=\"server\"></{0}:TwoRowLayout>"),
	ParseChildren(true)
	]
	public class TwoRowLayout : MyWebControl
	{

		#region Fields

		private ITemplate _topPlaceHolder = null;
		private ITemplate _bottomPlaceHolder = null;
		private ControlCollection _topPlaceHolderControls = null;
		private ControlCollection _bottomPlaceHolderControls = null;

		#endregion Fields

		#region Constructors

		/// <summary>
		/// Initializises a new instance of the TwoRowLayout class.
		/// </summary>
		public TwoRowLayout()
		{
		} 

		#endregion Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the template content for the place holder at the top of the control.
		/// </summary>
		[
		Browsable(false),
		PersistenceMode(PersistenceMode.InnerProperty), DefaultValue(null),
		//TemplateInstance(TemplateInstance.Single)
		]
		public virtual ITemplate TopPlaceHolder
		{
			get
			{
				return _topPlaceHolder;
			}
			set
			{
				_topPlaceHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the top place holder at run-time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection TopPlaceHolderControls
		{
			get
			{
				if (_topPlaceHolderControls == null)
				{
					_topPlaceHolderControls = new ControlCollection(this);
				}

				return _topPlaceHolderControls;
			}
		}

		/// <summary>
		/// Gets or sets the template content for the place holder at the bottom of the control.
		/// </summary>
		[
		Browsable(false),
		PersistenceMode(PersistenceMode.InnerProperty), DefaultValue(null),
		//TemplateInstance(TemplateInstance.Single)
		]
		public virtual ITemplate BottomPlaceHolder
		{
			get
			{
				return _bottomPlaceHolder;
			}
			set
			{
				_bottomPlaceHolder = value;
			}
		}

		/// <summary>
		/// Gets or sets the collection of controls to be added to the bottom place holder at run-time.
		/// Design-time controls are added first then run-time.
		/// </summary>
		[
		DefaultValue(null),
		Browsable(false)
		]
		public ControlCollection BottomPlaceHolderControls
		{
			get
			{
				if (_bottomPlaceHolderControls == null)
				{
					_bottomPlaceHolderControls = new ControlCollection(this);
				}

				return _bottomPlaceHolderControls;
			}
		}

		#endregion Properties

		#region Implementation Properties

		/// <summary>
		/// Gets the control's tag type.
		/// </summary>
		protected override HtmlTextWriterTag TagKey
		{
			get
			{
				return HtmlTextWriterTag.Div;
			}
		}

		#endregion Implementation Properties

		#region Implementation Methods

		/// <summary>
		/// Ensure child controls are available for referencing in the page load.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit(e);

			EnsureChildControls();
		}

		/// <summary>
		/// Ensure we add runtime controls so they are available for event handler re-attachment.
		/// </summary>
		/// <param name="e">
		/// The event data.
		/// </param>
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);

			if (!this.DesignMode)
			{
				AddRuntimeControls();
			}
		}

		/// <summary>
		/// Creates the control's layout.
		/// </summary>
		protected override void CreateChildControls()
		{
			this.Controls.Clear();

			this.Attributes.Add("class", "ribbonGroupBlock");

			Table table = new Table();
			table.CellSpacing = 0;
			table.CellPadding = 0;

			TableRow topRow = new TableRow();
			TableRow bottomRow = new TableRow();

			TableCell topCell = new TableCell();
			topCell.Attributes.Add("class", "ribbonGroupBlockHalf ribbonPart");

			TableCell bottomCell = new TableCell();
			bottomCell.Attributes.Add("class", "ribbonGroupBlockHalf ribbonPart");

			topRow.Cells.Add(topCell);
			bottomRow.Cells.Add(bottomCell);

			if (this.TopPlaceHolder != null)
			{
				this.TopPlaceHolder.InstantiateIn(topCell);
			}

			if (this.BottomPlaceHolder != null)
			{
				this.BottomPlaceHolder.InstantiateIn(bottomCell);
			}

			table.Rows.Add(topRow);
			table.Rows.Add(bottomRow);

			if (this.DesignMode)
			{
				table.Width = 8;
			}

			this.Controls.Add(table);
		}

		/// <summary>
		/// Adds controls added at run-time before rendering.
		/// </summary>
		private void AddRuntimeControls()
		{
			Table table = this.Controls[0] as Table;
			TableCell topCell = table.Rows[0].Cells[0];
			TableCell bottomCell = table.Rows[1].Cells[0];

			foreach (Control control in this.TopPlaceHolderControls)
			{
				topCell.Controls.Add(control);
			}

			foreach (Control control in this.BottomPlaceHolderControls)
			{
				bottomCell.Controls.Add(control);
			}
		}

		#endregion Implementation Methods

	}

}

